from django.contrib.gis.geos import Polygon, Point
from django.contrib.gis.measure import Distance
from math import pow, sqrt, asin, sin, cos, radians, degrees, pi
from geopy import Point as gpPoint
from geopy.distance import VincentyDistance

EARTH_RADIUS = Distance(km=6371)

def haversin(theta):
    return pow(sin(theta/2),2)

def ahaversin(haversine):
    return 2 * asin(sqrt(haversine))

def haversine_distance(lat1, lon1, lat2, lon2):
    #convert to radians
    lat1 = radians(lat1)
    lon1 = radians(lon1)
    lat2 = radians(lat2)
    lon2 = radians(lon2)
    diff_lat = lat2-lat1
    diff_lon = lon2-lon1
    haversine = haversin(diff_lat) + cos(lat1) * cos(lat2) * haversin(diff_lon)
    assert haversine >= 0 and haversine <= 1
    #get distance from haversine
    return EARTH_RADIUS * ahaversin(haversine)


def circle_as_polygon(lat, lon, n = 12, distance = Distance(km=1)):
    #this can be done all in degrees because that's what geopy uses!
    points = []
    #build points, loop around circle
    for angle in [360.*i/n for i in range(0,n)]:
        curr = VincentyDistance(kilometers=distance.km).destination(gpPoint(lat, lon), angle)
        points.append((curr.longitude, curr.latitude))
    #loop back to the first point for the n+1th point in the list
    points.append(points[0])
    return Polygon(tuple(points))



from django.contrib import admin
from gis_csdt.models import Dataset, MapPoint, PhoneNumber, Tag, MapPolygon, TagIndiv, DataField, DataElement, Sensor, DataPoint

class DatasetAdmin(admin.ModelAdmin):
    fieldsets = [
        (None,               {'fields': ['name','url','cache_max_age']}),
        ('Field Mapping',	{'fields': ['remote_id_field','name_field','lat_field','lon_field',
        							'street_field','city_field','state_field','zipcode_field',
        							'county_field']}),
        ('Custom Fields',	{'fields': ['field1_en','field1_name','field2_en','field2_name','field3_en','field3_name']}),
        ('Geocoding',		{'fields': ['needs_geocoding']})]

admin.site.register(Dataset, DatasetAdmin)
#for testing
admin.site.register(MapPoint)
admin.site.register(Tag)
admin.site.register(MapPolygon)
admin.site.register(TagIndiv)
admin.site.register(DataField)
admin.site.register(DataElement)
admin.site.register(Sensor)
admin.site.register(DataPoint)
admin.site.register(PhoneNumber)



from django.dispatch import receiver
from django.db.models.signals import post_save, pre_save
from gis_csdt.models import Dataset, Tag, TagIndiv

#@receiver(post_save, sender=Dataset)
def update_mappoints(sender, **kwargs):
	dataset = kwargs.get('instance')
	dataset.update_mappoints()	
#post_save.connect(update_mappoints, sender=Dataset)

@receiver(post_save, sender=TagIndiv)
def update_tag_count(sender, **kwargs):
	if kwargs.get('created'): #if this is a new TagIndiv
		#increment the count and save
		kwargs.get('instance').tag.increment_count(save=True)

@receiver(post_save, sender=Tag)
def clean_tags(sender, **kwargs):
	#all tags must be lowercase without leading or trailing whitespace
	kwargs.get('instance').tag = kwargs.get('instance').tag.strip().lower()
	


from gis_csdt.models import Dataset, MapElement, MapPoint, Tag, TagIndiv, MapPolygon, DataField, DataElement, Observation, ObservationValue, Sensor, DataPoint
from gis_csdt.filter_tools import filter_request, neighboring_points, unite_radius_bubbles
from gis_csdt.geometry_tools import circle_as_polygon
from gis_csdt.settings import CENSUS_API_KEY
from rest_framework import serializers, exceptions
from rest_framework_gis import serializers as gis_serializers
from django.contrib.gis.measure import Distance, Area
from django.contrib.gis.geos import GEOSGeometry
from django.core.exceptions import ObjectDoesNotExist
from django.core import serializers as model_serializers
from django.forms.models import model_to_dict
from django.db.models import Sum, Count
from django.http import HttpResponse, HttpRequest, HttpResponseBadRequest, HttpResponseNotAllowed, JsonResponse
from django import VERSION as DJANGO_VERSION
import copy, json, urllib

CIRCLE_EDGES = 12 #number of edges on polygon estimation of a circle
if DJANGO_VERSION[0] >= 1 and DJANGO_VERSION[1] >= 7:
    def get_function(function_name):
        if function_name[:4] == 'get_':
            return
else:
    def get_function(function_name):
        return function_name

'''class TagSerializer(serializers.ModelSerializer):
    def __init__(self, *args, **kwargs):
        many = kwargs.pop('many', True)
        super(TagSerializer, self).__init__(many=many, *args, **kwargs)

    class Meta:
        model = TagIndiv
        fields = ('mappoint','mappolygon','tag')'''

'''class MapElementIdField(serializers.WritableField):
    def __init__(self, *args,**kwargs):
        self.field = kwargs.pop('field')
        super(MapElementIdField, self).__init__(*args, **kwargs)
    def to_native(self, obj):
        try:
            r = getattr(obj,self.field)
            return r.id
        except ObjectDoesNotExist:
            return None
    def from_native(self, data):
        try:
            return MapElement.objects.get(id=int(data))
        except (ObjectDoesNotExist, ValueError):
            return None'''

class NewTagSerializer(serializers.ModelSerializer):
    #mappolygon = MapElementIdField(required=False, source='mapelement_id', field='mappolygon')
    mapelement = serializers.IntegerField(source='mapelement_id')
    tag = serializers.CharField()


    class Meta:
        model = TagIndiv
        fields = ('mapelement','tag')

    def validate_tag(self, value):
        if ',' in value or type(value) is list:
            raise serializers.ValidationError('No more than one tag should be POSTed at once. Commas are not allowed in tags.')
        return value.strip().lower()

    def create(self, validated_data):
        me = validated_data.pop('mapelement_id')
        try:
            me = MapElement.objects.get(id=me)
        except:
            raise serializers.ValidationError('No MapElement found with ID',str(value))
        tag_txt = validated_data.pop('tag')
        tags = Tag.objects.filter(dataset = me.dataset, tag = tag_txt)

        len_tags = len(tags)
        if len_tags == 0:
            tag = Tag(dataset = me.dataset, tag = tag_txt)
            tag.save()
        elif len_tags == 1:
            tag = tags[0]
        else:
            approved_tags = tags.filter(approved = True)
            if len(approved_tags) > 0:
                tag = approved_tags[0]
            else:
                tag = tags[0]
        return TagIndiv.objects.create(mapelement=me, tag=tag)

class TagCountSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Tag 
        fields = ['dataset','tag','count']

class DatasetSerializer(serializers.ModelSerializer):
    tags = serializers.SerializerMethodField(get_function('get_tags'))
    count = serializers.SerializerMethodField(get_function('get_count'))
    def get_tags(self, dataset):
        #build nested distinct list
        return Tag.objects.filter(approved=True, dataset=dataset).order_by('-count').values_list('tag',flat=True)
    def get_count(self, dataset):
        #build nested distinct list
        return MapElement.objects.filter(dataset=dataset).count()


    class Meta:
        model = Dataset
        fields = ('id','name','cached','count','field1_en','field2_en','field3_en', 'tags')

class MapPointSerializer(serializers.HyperlinkedModelSerializer):
    latitude = serializers.DecimalField(source = 'mappoint.lat', max_digits=18, decimal_places=15)
    longitude = serializers.DecimalField(source = 'mappoint.lon', max_digits=18, decimal_places=15)
    street = serializers.CharField(source = 'mappoint.street')
    city = serializers.CharField(source = 'mappoint.city')
    state = serializers.CharField(source = 'mappoint.state')
    zipcode = serializers.CharField(source = 'mappoint.zipcode')
    county = serializers.CharField(source = 'mappoint.county')
    field1 = serializers.CharField(source = 'mappoint.field1')
    field2 = serializers.CharField(source = 'mappoint.field2')
    field3 = serializers.CharField(source = 'mappoint.field3')

    tags = serializers.SerializerMethodField(get_function('get_tags'))
    def get_tags(self, mappoint):
        #build nested distinct list
        return Tag.objects.filter(approved=True, tagindiv__mapelement=mappoint).distinct('id','tag').values_list('tag',flat=True)

    class Meta:
        #id_field = False
        #geo_field = 'point'
        model = MapPoint 
        fields = ('dataset','id','name','latitude','longitude','street','city','state','zipcode','county','field1','field2','field3','tags')

class TestSerializer(gis_serializers.GeoFeatureModelSerializer):
    address = serializers.SerializerMethodField(get_function('get_address'))
    data = serializers.SerializerMethodField(get_function('get_data'))
    tags = serializers.SerializerMethodField(get_function('get_tags'))
    geom = serializers.SerializerMethodField(get_function('get_geom'))

    def get_address(self, mapelement):
        if hasattr(mapelement,'mappoint'):
            return {'street':mapelement.mappoint.street,'city':mapelement.mappoint.city,'state':mapelement.mappoint.state,'zipcode':mapelement.mappoint.zipcode,'county':mapelement.mappoint.county}
        return {}

    def get_data(self, mapelement):
        data = {}
        child = mapelement.mappoint if hasattr(mapelement,'mappoint') else mapelement.mappolygon
        dataset = Dataset.objects.get(id=mapelement.dataset_id)
        dataelements = DataElement.objects.filter(mapelement_id=mapelement.id)

        if dataset.field1_name != '' and (child.field1 != '' or child.field1 is not None):
            data[dataset.field1_en] = child.field1
        if dataset.field2_name != '' and (child.field2 != '' or child.field2 is not None):
            data[dataset.field2_en] = child.field2
        if hasattr(mapelement,'mappoint') and dataset.field3_name != '' and (child.field3 != '' or child.field3 is not None):
            data[dataset.field3_en] = child.field3

        params = self.context.get('request', None).QUERY_PARAMS
        if 'data' in params and params['data'] == 'all':
            for de in dataelements:
                for ft, field_name in {DataField.INTEGER: 'int_data', DataField.FLOAT: 'float_data', DataField.STRING: 'char_data'}.iteritems():
                    if de.datafield.field_type == ft:
                        data[de.datafield.field_en] = getattr(de,field_name)

        return data

    def get_tags(self, mapelement):
        #build nested distinct list
        return Tag.objects.filter(approved=True, tagindiv__mapelement=mapelement).distinct('tag').values_list('tag',flat=True)

    def get_geom(self,mapelement):
        if hasattr(mapelement,'mappolygon'):
            return json.loads(GEOSGeometry(mapelement.mappolygon.mpoly).geojson)
        return json.loads(GEOSGeometry(mapelement.point).geojson)

    class Meta:
        id_field = 'id'
        geo_field = 'geom'
        model = MapElement 
        fields = ('dataset','id','name','address','data','tags')

class MapPolygonSerializer(gis_serializers.GeoFeatureModelSerializer):
    latitude = serializers.DecimalField(source = 'mappolygon.lat', max_digits=18, decimal_places=15)
    longitude = serializers.DecimalField(source = 'mappolygon.lon', max_digits=18, decimal_places=15)
    field1 = serializers.CharField(source = 'mappolygon.field1')
    field2 = serializers.CharField(source = 'mappolygon.field2')
    
    mpoly = gis_serializers.GeometryField(source = 'mappolygon.mpoly')

    tags = serializers.SerializerMethodField(get_function('get_tags'))

    def get_tags(self, mappolygon):
        #build nested distinct list
        return Tag.objects.filter(approved=True, tagindiv__mapelement=mappolygon).distinct('id','tag').values_list('tag',flat=True)

    class Meta:
        id_field = False
        geo_field = 'mpoly'
        model = MapPolygon 
        fields = ('id','dataset','remote_id','name','latitude','longitude','field1','field2','tags')

class CountPointsSerializer(serializers.ModelSerializer):
    polygon_id = serializers.IntegerField(source = 'remote_id')
    count = serializers.SerializerMethodField(get_function('get_count'))

    class Meta:
        model = MapPolygon
        fields = ('polygon_id','count')

    def get_count(self, mappolygon):
        request = self.context.get('request', None)
        params = copy.deepcopy(request.QUERY_PARAMS)
        for key in ['max_lat','min_lat','max_lon','min_lon','state']:
            try:
                del params[key]
            except:
                pass #no big deal

        c = {mappolygon.dataset.field1_en : mappolygon.mappolygon.field1, mappolygon.dataset.field2_en : mappolygon.mappolygon.field2}

        datafields = DataField.objects.filter(dataset=mappolygon.dataset)
        #get other data
        for df in datafields:
            data = None
            if df.field_type == DataField.INTEGER:
                element = DataElement.objects.filter(datafield = df).filter(mapelement=mappolygon)
                if element:
                    data = element[0].int_data
            elif df.field_type == DataField.FLOAT:
                element = DataElement.objects.filter(datafield = df).filter(mapelement=mappolygon)
                if element:
                    data = element[0].float_data
            else:
                element = DataElement.objects.filter(datafield = df).filter(mapelement=mappolygon)
                if element:
                    data = element[0].char_data
            if data:
                c[df.field_en] = data
        
        points = filter_request(params,'mappoint').filter(point__intersects=mappolygon.mappolygon.mpoly)
        all_tags = None
        if 'tag' in params:
            all_tags = params['tag']
        elif 'tags' in params:
            all_tags = params['tags']
        if all_tags:
            tags = all_tags.split(',')
            if type(tags) is not list:
                tags = [tags]
            all_tags = all_tags.replace(',',', ')
        else:
            tags = []

        #counts in polygons
        if 'match' not in params or params['match'] != 'all':
            all_tag_filter = points
            for tag in tags:
                try:
                    num = int(tag)
                    tag_obj = Tag.objects.get(num)
                    all_tag_filter = all_tag_filter.filter(tagindiv__tag=tag_obj)
                    c[tag_obj.tag + " count"] = points.filter(tagindiv__tag=tag_obj).count()
                except:
                    all_tag_filter = all_tag_filter.filter(tagindiv__tag__tag=tag)
                    c[tag + " count"] = points.filter(tagindiv__tag__tag=tag).count()
            if len(tags) > 1:
                c[all_tags + " count (match any)"] = points.count()
                c[all_tags + " count (match all)"] = points.count()
        if len(tags) > 1:
            c[all_tags + " count (match all)"] = points.count()

        return c

class AnalyzeAreaSerializer(serializers.ModelSerializer):
    street = serializers.CharField(source = 'mappoint.street')
    city = serializers.CharField(source = 'mappoint.city')
    state = serializers.CharField(source = 'mappoint.state')
    zipcode = serializers.CharField(source = 'mappoint.zipcode')
    county = serializers.CharField(source = 'mappoint.county')
    field1 = serializers.CharField(source = 'mappoint.field1')
    field2 = serializers.CharField(source = 'mappoint.field2')
    field3 = serializers.CharField(source = 'mappoint.field3')

    tags = serializers.SerializerMethodField(get_function('get_tags'))
    areaAroundPoint = serializers.SerializerMethodField(get_function('get_areaAroundPoint'))

    class Meta:
        model = MapPoint 
        fields = ('street','city','state','zipcode','county','field1','field2','field3','tags','areaAroundPoint')

    def get_tags(self, mappoint):
        #build nested distinct list
        return Tag.objects.filter(approved=True, tagindiv__mapelement=mappoint).distinct('tag').values('tag')
    
    def get_areaAroundPoint(self, mappoint):
        request = self.context.get('request', None)

        years = request.GET.getlist('year')
        datasets = Dataset.objects.filter(name__icontains='census')
        if len(years) > 0:
            d = Dataset.objects.none()
            for y in years:
                d = d | datasets.filter(name__contains=y.strip())
            datasets = d
        if len(datasets) == 0:
            return {}
        else:
            dataset_id = datasets[0].id

        ### DISTANCES
        distances = request.GET.getlist('distance')
        unit = request.GET.getlist('unit')
        if len(unit) > 1:
            return HttpResponseBadRequest('No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m','km','mi']:
            unit = unit[0]
        else:
            return HttpResponseBadRequest('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1,3,5]
            unit = 'km'
        else:
            distances.sort()

        dist_objs = []
        for dist in distances:
            kwargs = {unit: dist}
            dist_objs.append(Distance(**kwargs))

        all_points = neighboring_points(mappoint, MapPoint.objects.filter(dataset_id=mappoint.dataset_id), dist_objs[-1])


        data_sums = {'point id(s)':'', 'view url(s)':[]}
        for p in all_points:
            data_sums['point id(s)'] = data_sums['point id(s)'] + ',' + str(p.id)
            data_sums['view url(s)'].append('/around-point/%d/' %(p.id))
        data_sums['view url(s)'] = str(data_sums['view url(s)'])
        data_sums['point id(s)'] = data_sums['point id(s)'].strip(',')
        already_accounted = set()
        boundary = unite_radius_bubbles(all_points,dist_objs)
        for dist in dist_objs:
            if unit == 'm':
                dist_str = '%f m' %(dist.m)
            elif unit == 'km':
                dist_str = '%f km' %(dist.km)
            elif unit == 'mi':
                dist_str = '%f mi' %(dist.mi)
            poly = set(MapPolygon.objects.filter(dataset_id__exact=dataset_id).filter(mpoly__covers=boundary[dist]).exclude(remote_id__in=already_accounted).values_list('remote_id',flat=True))
            maybe_polys = MapPolygon.objects.filter(dataset_id__exact=dataset_id).exclude(mpoly__covers=boundary[dist]).exclude(remote_id__in=already_accounted).filter(mpoly__intersects=boundary[dist])
            for polygon in maybe_polys:
                if polygon.mpoly.intersection(boundary[dist]).area > .5 * polygon.mpoly.area:
                    poly.add(polygon.remote_id)
            already_accounted = already_accounted | poly
            data_sums[dist_str] = {}
            data_sums[dist_str]['polygon count'] = len(poly)
            data_sums[dist_str]['land area (m2)'] = sum([int(i) for i in MapPolygon.objects.filter(dataset_id__exact=dataset_id,remote_id__in=poly).values_list('field1',flat=True)])
            if data_sums[dist_str]['polygon count'] > 0:
                data_sums[dist_str]['polygons'] = str(list(poly))
                datafields = DataField.objects.filter(dataset_id__exact=dataset_id).exclude(field_type__exact=DataField.STRING)
                for field in datafields:
                    if field.field_longname not in data_sums[dist_str]:
                        data_sums[dist_str][field.field_longname] = {}
                    if field.field_type == DataField.INTEGER:
                        data = DataElement.objects.filter(datafield_id=field.id,mapelement__remote_id__in=poly).aggregate(sum=Sum('int_data'),dsum=Sum('denominator__int_data'))
                        #print data['sum'], data['dsum']
                    elif field.field_type == DataField.FLOAT:
                        data = DataElement.objects.filter(datafield_id=field.id,mapelement__remote_id__in=poly).aggregate(sum=Sum('float_data'),dsum=Sum('denominator__float_data'))
                    else:
                        continue
                    data_sums[dist_str][field.field_longname][field.field_en] = data['sum']
                    data_sums[dist_str][field.field_longname]['total']= data['dsum']

        return data_sums

    def area_around_point2(self, mappoint):
        request = self.context.get('request', None)

        years = request.GET.getlist('year')
        year = years[0]
        datasets = Dataset.objects.filter(name__icontains='census')
        if len(years) > 0:
            d = Dataset.objects.none()
            for y in years:
                d = d | datasets.filter(name__contains=y.strip())
            datasets = d
        if len(datasets) == 0:
            return {}
        else:
            dataset_id = datasets[0].id

        ### DISTANCES
        distances = request.GET.getlist('distance')
        unit = request.GET.getlist('unit')
        if len(unit) > 1:
            return HttpResponseBadRequest('No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m','km','mi']:
            unit = unit[0]
        else:
            return HttpResponseBadRequest('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1,3,5]
            unit = 'km'
        else:
            distances.sort()

        dist_objs = []
        for dist in distances:
            kwargs = {unit: dist}
            dist_objs.append(Distance(**kwargs))

        all_points = neighboring_points(mappoint, MapPoint.objects.filter(dataset=mappoint.dataset_id), dist_objs[-1])
        #print all_points

        #variables = ['Total Population','Area (km2)','Total (Race)', 'White Only', 'African American', 'Hispanic','Asian/Pacific Islander', 'Native American','Total (Poverty)','below 1.00', 'weighted mean of median household income','Mean Housing Value']
        #variables = {'B02001_001E':{},'B02001_002E':{},'B02009_001E':{},'B03001_001E':{},'B03001_003E':{},'B02011_001E':{}, 'B02012_001E':{},'B02010_001E':{},'B05010_001E':{},'B05010_002E':{},'B19061_001E':{},'B25105_001E':{},'B25077_001E':{},'B25077_001E':{}}
        variables = {'B00001_001E':{},'B02001_001E':{},'B02001_002E':{},'B02001_003E':{},'B02001_004E':{},'B02001_005E':{},'B02001_006E':{},'B02001_007E':{},'B02001_008E':{},'B03001_001E':{},'B03001_003E':{},'C17002_001E':{},'C17002_002E':{},'C17002_003E':{}}
        for v in variables:
            request = 'http://api.census.gov/data/2010/acs5/variables/%s.json?key=%s' %(v,CENSUS_API_KEY)
            try:
                data = json.loads(urllib.urlopen(request).read())
            except Exception as e:
                print 'variable info for %s failed to load: %s'%(v,request)
                print e
                continue
            variables[v] = data


        data_sums = {'point id(s)':'', 'view url(s)':[]}
        for p in all_points:
            data_sums['point id(s)'] = data_sums['point id(s)'] + ',' + str(p.id)
            data_sums['view url(s)'].append('/around-point/%d/' %(p.id))
        data_sums['point id(s)'] = data_sums['point id(s)'].strip(',')
        already_accounted = set()

        boundary = unite_radius_bubbles(all_points,dist_objs)
        for dist in dist_objs:
            if unit == 'm':
                dist_str = '%f m' %(dist.m)
            elif unit == 'km':
                dist_str = '%f km' %(dist.km)
            elif unit == 'mi':
                dist_str = '%f mi' %(dist.mi)
            poly = set(MapPolygon.objects.filter(dataset_id__exact=dataset_id).filter(mpoly__covers=boundary[dist]).exclude(remote_id__in=already_accounted).values_list('remote_id',flat=True))
            maybe_polys = MapPolygon.objects.filter(dataset_id__exact=dataset_id).exclude(mpoly__covers=boundary[dist]).exclude(remote_id__in=already_accounted).filter(mpoly__intersects=boundary[dist])
            for polygon in maybe_polys:
                if polygon.mpoly.intersection(boundary[dist]).area > .5 * polygon.mpoly.area:
                    poly.add(polygon.remote_id)
            already_accounted = already_accounted | poly
            data_sums[dist_str] = {}
            data_sums[dist_str]['polygon count'] = len(poly)
            data_sums[dist_str]['land area (m2)'] = sum([int(i) for i in MapPolygon.objects.filter(dataset_id__exact=dataset_id,remote_id__in=poly).values_list('field1',flat=True)])
            if data_sums[dist_str]['polygon count'] > 0:
                data_sums[dist_str]['polygons'] = str(list(poly))
                if year == '2010':
                    get = ''
                    for e in variables:
                        get = get + ',' + e
                    get = get.strip(',')
                    tracts = {}
                    for e in poly:
                        if e[:2] not in tracts:
                            tracts[e[:2]] = {}
                        if e[2:5] not in tracts[e[:2]]:
                            tracts[e[:2]][e[2:5]]=''
                        tracts[e[:2]][e[2:5]] = tracts[e[:2]][e[2:5]] + ',' + e[-6:]
                    for state in tracts:
                        for county in tracts[state]:
                            request = 'http://api.census.gov/data/2010/acs5?key=%s&get=%s,NAME&for=tract:%s&in=state:%s,county:%s' %(CENSUS_API_KEY,get,tracts[state][county].strip(','),state,county)
                            try:
                                url = urllib.urlopen(request).read()
                                data = json.loads(url)
                            except Exception as e:
                                print e
                                print url
                                print request
                                continue
                            line = data[0]
                            locations = {}
                            for i in range(len(line)):
                                locations[line[i]] = i
                                if line[i] not in data_sums[dist_str] and line[i] in variables:
                                    data_sums[dist_str][line[i]] = copy.deepcopy(variables[line[i]])
                                    data_sums[dist_str][line[i]]['sum'] = 0

                            for line in data[1:]:
                                #get area
                                for v in variables:
                                    try:
                                        data_sums[dist_str][v]['sum'] = data_sums[dist_str][v]['sum'] + int(line[locations[v]])
                                    except:
                                        continue

        return data_sum

class SensorSerializer(serializers.ModelSerializer):
    name = serializers.CharField(max_length=100)
    supplier = serializers.CharField(max_length=100)
    model_number = serializers.CharField(max_length=100)
    metric = serializers.CharField(max_length=100)
    accuracy = serializers.CharField(max_length=100)
    class Meta:
        model = Sensor
        fields = ('name', 'supplier', 'model_number', 'metric', 'accuracy')
    def create(self, attrs, instance=None):
        sensorModel = Sensor(name=attrs['name'].strip(),supplier=attrs['supplier'].strip(),model_number=attrs['model_number'],metric=attrs['metric'],accuracy=attrs['accuracy'])
        sensorModel.save()
        return sensorModel

class ObservationSerializer(serializers.ModelSerializer):
    class Meta:
        model = Observation
        fields = ('mapelement','time','accuracy')
    def restore_object(self, attrs, instance=None):
        try:
            return Observation.objects.get(mapelement_id=attrs['mapelement_id'],time=attrs['time'],accuracy=attrs['accuracy'])
        except:
            return Observation(mapelement_id=attrs['mapelement_id'],time=attrs['time'],accuracy=attrs['accuracy'])

class SensedDataSerializer(serializers.ModelSerializer):
    sensor_name = serializers.CharField(source='observation__sensor__name')
    sensor_type = serializers.CharField()
    mapelement = serializers.IntegerField()
    time = serializers.DateTimeField()
    accuracy = serializers.FloatField(required=False)
    value_name = serializers.CharField(source='name')
    value = serializers.FloatField()

    class Meta:
        model = ObservationValue
        fields = ('sensor_name','sensor_type','mapelement','time','accuracy','value_name','value')

    def create(self, attrs, instance=None):
        #check that we have enough info
        #if not set('sensor_name','sensor_type','mapelement','time','value_name','value').issubset(set(attrs.keys())):
        #    raise exceptions.ParseError()
        #find the mappoint
        try:
            mp = MapElement.objects.get(id=attrs['mapelement'])
        except:
            raise exceptions.ParseError()
        #find or create the sensor
        sensor = Sensor.objects.filter(name__iexact=attrs['observation__sensor__name'].strip(),sensor_type__iexact=attrs['sensor_type'].strip())
        if len(sensor)==0:
            sensor = Sensor(name=attrs['observation__sensor__name'].strip(),sensor_type=attrs['sensor_type'].strip())
            sensor.save()
        else:
            sensor = sensor[0]
        #find or create the observation
        observation = Observation.objects.filter(mapelement_id=mp.id,sensor_id=sensor.id,time__exact=attrs['time'],accuracy__exact=attrs['accuracy'])
        if len(observation)==0:
            observation = Observation(mapelement=mp,sensor=sensor,time=attrs['time'],accuracy=attrs['accuracy'])
            observation.save()
        else:
            observation = observation[0]
        #create the value
        return ObservationValue(observation=observation,name=attrs['name'].strip(),value=attrs['value'])

class DataPointSerializer(serializers.ModelSerializer):
    value = serializers.DecimalField(max_digits=30, decimal_places=15)
    sensors = SensorSerializer(many=True, read_only=True)
    points = MapPointSerializer(many=True, read_only=True)

    class Meta:
        model = DataPoint
        fields = ('__all__')

    def create(self, attrs, instance=None):
        thisUser = self.context['request'].user
        datapointModel = DataPoint(value=attrs['value'], sensor=attrs['sensor'], point=attrs['point'], user_id=thisUser.id)
        datapointModel.save()
        return datapointModel

class AnalyzeAreaNoValuesSerializer(serializers.ModelSerializer):
    point_id = serializers.CharField(source = 'mappoint.id')
    areaAroundPoint = serializers.SerializerMethodField(get_function('get_areaAroundPoint'))

    class Meta:
        model = MapPoint 
        fields = ('point_id','areaAroundPoint')


    def get_tags(self, mappoint):
        #build nested distinct list
        return Tag.objects.filter(approved=True, tagindiv__mapelement=mappoint).distinct('tag').values('tag')
    
    def get_areaAroundPoint(self, mappoint):
        request = self.context.get('request', None)
        years = request.GET.getlist('year')
        datasets = Dataset.objects.filter(name__icontains='census')
        if len(years) > 0:
            d = Dataset.objects.none()
            for y in years:
                d = d | datasets.filter(name__contains=y.strip())
            datasets = d
        if len(datasets) == 0:
            return {}
        else:
            dataset_id = datasets[0].id

        ### DISTANCES
        distances = request.GET.getlist('distance')
        unit = request.GET.getlist('unit')
        if len(unit) > 1:
            return HttpResponseBadRequest('No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m','km','mi']:
            unit = unit[0]
        else:
            return HttpResponseBadRequest('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1,3,5]
            unit = 'km'
        else:
            distances.sort()

        #method is containment or apportionment?
        method = request.GET.getlist('method')
        if len(method) > 0 and method[0] == 'apportionment':
            apportionment = True
        else:
            apportionment = False

        dist_objs = []
        for dist in distances:
            kwargs = {unit: dist}
            dist_objs.append(Distance(**kwargs))

        all_points = neighboring_points(mappoint, MapPoint.objects.filter(dataset_id=mappoint.dataset_id), dist_objs[-1])
        #print len(all_points), all_points
        data_sums = {'points':[], 'view url':'http://127.0.0.1:8000/around-point/%d/' %(all_points[0].id)}
        for p in all_points:
            data_sums['points'].append({'id':p.id,
                                        'name':p.name, 
                                        'street':p.mappoint.street, 
                                        'city':p.mappoint.city, 
                                        'state':p.mappoint.state, 
                                        'zipcode':p.mappoint.zipcode, 
                                        'county':p.mappoint.county, 
                                        'field1':p.mappoint.field1, 
                                        'field2':p.mappoint.field2, 
                                        'field3':p.mappoint.field3, 
                                        'tags':Tag.objects.filter(approved=True, tagindiv__mapelement=mappoint).distinct('tag').values('tag')})
        already_accounted = set()
        boundary = unite_radius_bubbles(all_points,dist_objs)
        for dist in dist_objs:
            if unit == 'm':
                dist_str = '%f m' %(dist.m)
            elif unit == 'km':
                dist_str = '%f km' %(dist.km)
            elif unit == 'mi':
                dist_str = '%f mi' %(dist.mi)
            temp_qs = MapPolygon.objects.filter(dataset_id__exact=dataset_id).filter(mpoly__coveredby=boundary[dist]).exclude(remote_id__in=already_accounted).distinct()
            poly = {x: 1. for x in temp_qs.values_list('remote_id',flat=True)}
            try:
                land_area = sum([int(x) for x in temp_qs.values_list('field1',flat=True)])
            except:
                continue
            #print 'poly', poly
            maybe_polys = MapPolygon.objects.filter(dataset_id__exact=dataset_id).exclude(remote_id__in=poly).exclude(remote_id__in=already_accounted).filter(mpoly__intersects=boundary[dist]).distinct()
            #print 'maybes', maybe_polys.values_list('remote_id',flat=True)
            for polygon in maybe_polys:
                intersection_area = polygon.mpoly.intersection(boundary[dist]).area / polygon.mpoly.area
                if apportionment or intersection_area > .5:
                    poly[polygon.remote_id] = intersection_area if apportionment else 1.
                    try:
                        land_area += intersection_area*int(polygon.field1) if apportionment else int(polygon.field1)
                    except:
                        continue

            already_accounted = already_accounted | set(poly.keys())
            data_sums[dist_str] = {}
            # data_sums[dist_str]['polygon count'] = len(poly)
            # data_sums[dist_str]['land area (m2)'] = sum([int(i) for i in MapPolygon.objects.filter(dataset_id__exact=dataset_id,remote_id__in=poly).values_list('field1',flat=True)])
            if len(poly) > 0:
                data_sums[dist_str]['polygons'] = poly
                data_sums[dist_str]['land_area'] = land_area





# Django settings for gis project.
import os

PROJECT_ROOT = os.path.dirname(os.path.dirname(__file__))

DEBUG = True # False
ALLOWED_HOSTS = ['*'] ##FOR TESTING ONLY

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
#        'ENGINE': 'django.contrib.gis.db.backends.postgis',
#        'ENGINE': 'django.db.backends.postgresql',
	'ENGINE': 'django.contrib.gis.db.backends.postgis',
        'NAME': 'django_test',
        'USER': 'django_user',
        'PASSWORD': 'dj4ng0_t3st',
        'HOST': 'localhost',
        'PORT': '5432',
    }
}


# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'America/New_York'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/var/www/example.com/media/"
MEDIA_ROOT = PROJECT_ROOT + '/media/'

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://example.com/media/", "http://media.example.com/"
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = PROJECT_ROOT + '/collected_static/'

# URL prefix for static files.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    PROJECT_ROOT + '/static/',
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    # 'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = '39=xae5za1g4(*#9o05-gzqbp07qec(r4!p)^m7yvtu4!z2@3p'

# List of callables that know how to import templates from various sources.

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'gis_csdt.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'gis_csdt.wsgi.application'


INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.sites',
    'django.contrib.admindocs',
    'rest_framework',
    'django_extensions',
    'django.contrib.gis',
    'gis_csdt',
)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

REST_FRAMEWORK = {
    # Use hyperlinked styles by default.
    # Only used if the `serializer_class` attribute is not set on a view.
    'DEFAULT_MODEL_SERIALIZER_CLASS':
        'rest_framework.serializers.HyperlinkedModelSerializer',

    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly'
    ],
    #'EXCEPTION_HANDLER': 'gis_csdt.exceptions.custom_exception_handler',
    'EXCEPTION_HANDLER': 'gis_csdt.exceptions.custom_exception_handler',
    'TEST_REQUEST_DEFAULT_FORMAT': 'json'
}


TEMPLATES = [ 
    {   
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(PROJECT_ROOT, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],  
        },  
    },  
]

try:
    from local_settings import *
except ImportError:
    GOOGLE_API_KEY = None
    CENSUS_API_KEY = None



from rest_framework import status
from rest_framework.test import APITestCase, APIClient
from django.core.urlresolvers import reverse
from gis_csdt.models import Dataset, MapPoint, Sensor, DataPoint, PhoneNumber
from gis_csdt.views import DataToGSM7
from django.contrib.auth import get_user_model
from django.test import LiveServerTestCase
import urllib
User = get_user_model()

class AllViewTestsNoData(APITestCase):
    def test_no_datasets(self):
        response = self.client.get('/api-ds/')
        self.assertEqual(response.status_code, 200)
    def test_no_mappoints(self):
        response = self.client.get('/api-mp/')
        self.assertEqual(response.status_code, 200)
    def test_no_newtags(self):
        response = self.client.get('/api-newtag/')
        self.assertEqual(response.status_code, 200)
    def test_no_polygons(self):
        response = self.client.get('/api-poly/')
        self.assertEqual(response.status_code, 200)
    def test_no_mappoints_geojson(self):
        response = self.client.get('/api-test/')
        self.assertEqual(response.status_code, 200)
    def test_no_mappolygons_count_of_points(self):
        response = self.client.get('/api-count/')
        self.assertEqual(response.status_code, 200)
    def test_no_mappolygons_analysis_around_point(self):
        response = self.client.get('/api-dist/')
        self.assertEqual(response.status_code, 400)

class SMSCreateData(LiveServerTestCase):
    def setUp(self):
        self.client = APIClient()
        self.user = User.objects.create_superuser(username='test',
                                                  email='test@test.test',
                                                  password='test')
        self.assertTrue(self.client.login(username='test', password='test'))

    def test_SMS(self):
        data = [1,1,0,128,129,300,10001]
        string = DataToGSM7(data)
        set = Dataset.objects.create(name='test')
        set.save()
        point = MapPoint.objects.create(lat=0,lon=0)
        point.save()
        sensor = Sensor.objects.create(name='test')
        sensor.save()
        phNum = PhoneNumber.objects.create(phone_number=11111111111,user=self.user)
        phNum.save()
        postData = {'Body': string.encode('utf-8'), 'From': phNum.phone_number}
        response = self.client.post('/api-SMS/', urllib.urlencode(postData), content_type='application/x-www-form-urlencoded')
        self.assertEqual(DataPoint.objects.get(pk=5).value, 10001)



from django.contrib.gis.geos import Polygon, Point
from gis_csdt.geometry_tools import circle_as_polygon
from gis_csdt.models import Dataset, MapElement, MapPoint, Tag, MapPolygon, TagIndiv, DataField, DataElement
from django.contrib.gis.measure import Distance
from django.db.models import Q

def filter_request(parameters, model_type):

    if 'tag' in parameters:
        tags = parameters['tag']
    elif 'tags' in parameters:
        tags = parameters['tags']
    else:
        tags = False

    if 'match' in parameters:
        matchall = parameters['match'] == 'all'
    else:
        matchall = False

    
    queryset = MapElement.objects.none()
    if tags:
        tags = tags.split(',')
        if type(tags) is not list:
            tags = [tags]
        if matchall:
            queryset = MapElement.objects
            for tag in tags:
                try:
                    num = int(tag)
                    queryset = queryset.filter(tagindiv__tag=num)
                except:
                    queryset = queryset.filter(tagindiv__tag__tag=tag)
        else:
            for tag in tags:
                try:
                    num = int(tag)
                    queryset = queryset | MapElement.objects.filter(tagindiv__tag=num)
                except:
                    queryset = queryset | MapElement.objects.filter(tagindiv__tag__tag=tag)
        queryset = queryset.filter(tagindiv__tag__approved=True)
    else:
        queryset = MapElement.objects

    if 'remote_id' in parameters:
        queryset = queryset.filter(remote_id=parameters['remote_id'])


    if 'dataset' in parameters:
        dataset_list = parameters['dataset'].strip().split(',')
        if type(dataset_list) is not list:
            dataset_list = [dataset_list]
        for dataset in dataset_list:
            try:
                r = int(dataset)
                queryset = queryset.filter(dataset__id__exact = r)
            except:
                queryset = queryset.filter(dataset__name__icontains = dataset)

    #make it just one type now
    if model_type != 'mapelement':
        kwargs = {model_type:None}
        queryset = queryset.exclude(**kwargs)

    if model_type == 'mappoint':
        if 'street' in parameters:
            queryset = queryset.filter(mappoint__street__iexact = parameters['street'])
        if 'city' in parameters:
            queryset = queryset.filter(mappoint__city__iexact = parameters['city'])
        if 'state' in parameters:
            queryset = queryset.filter(mappoint__state__iexact = parameters['state'])
        if 'county' in parameters:
            queryset = queryset.filter(mappoint__county__iexact = parameters['county'])
        for key in ['zipcode','zip','zip_code']:
            if key in parameters:
                queryset = queryset.filter(mappoint__zipcode__iexact = parameters[key])

    bb = {}
    for key in ['max_lat','min_lat','max_lon','min_lon']:
        if key in parameters:
            try:
                r = float(parameters[key])
            except:
                return HttpResponseBadRequest('Invalid radius. Only integers accepted.' )
                continue
            bb[key] = r

    if 'max_lat' in bb and 'min_lat' in bb and 'max_lon' in bb and 'min_lon' in bb:
        geom = Polygon.from_bbox((bb['min_lon'],bb['min_lat'],bb['max_lon'],bb['max_lat']))
        if model_type == 'mappoint':
            queryset = queryset.filter(point__within=geom)
        elif model_type == 'mappolygon':
            queryset = queryset.filter(mappolygon__mpoly__bboverlaps=geom)
        elif model_type == 'mapelement':
            queryset = queryset.filter((~Q(mappoint=None) & Q(point__within=geom)) | (~Q(mappolygon=None) & Q(mappolygon__mpoly__bboverlaps=geom)))

    if 'radius' in parameters and 'center' in parameters:
        try:
            radius = int(parameters['radius'])
        except:
            return HttpResponseBadRequest('Invalid radius. Only integers accepted.')
        temp = parameters.split(',')
        try:
            if len(temp) != 2:
                raise 
            temp[0] = float(temp[0])
            temp[1] = float(temp[1])
            center = Point(temp[0],temp[1])
        except:
            return HttpResponseBadRequest('Invalid center. Format is: center=lon,lat' )
        queryset = queryset.filter(point__distance_lte = (center,Distance(mi=radius)))
    elif 'radius' in parameters or 'center' in parameters:
        return HttpResponseBadRequest('If a center or radius is specified, the other must also be specified.')

    return queryset.distinct()

def neighboring_points(point, queryset, distance):
    distance = distance * 2
    all_points=queryset.filter(point__distance_lte=(point.point,distance)).distinct()
    point_set = list(all_points)#.values_list('point',flat=True))
    print point_set
    for p in point_set:
        new_points = queryset.filter(point__distance_lte=(p.point,distance)).exclude(id__in=[x.id for x in point_set]).distinct()
        all_points = all_points | new_points
        point_set.extend(list(new_points))#.values_list('point',flat=True)))
    return all_points.distinct()

def unite_radius_bubbles(points, distances):
    geometry = {}
    for d in distances:
        geometry[d] = circle_as_polygon(lat = points[0].point.y, lon = points[0].point.x, distance = d)
        for p in points[1:]:
            geometry[d] = geometry[d].union(circle_as_polygon(lat = p.point.y, lon = p.point.x, distance = d))
    return geometry


# -*- coding: utf8 -*-
import sys
import binascii
from rest_framework import views, viewsets, permissions, response, pagination
from rest_framework.settings import api_settings
from rest_framework_csv.renderers import CSVRenderer
from rest_framework.exceptions import ParseError #, APIException
from django.contrib.gis.db.models import Count
from django.contrib.gis.geos import Polygon, Point
from django.contrib.gis.measure import Distance, Area
from django.views.generic.edit import CreateView
from django.core.paginator import Paginator
from django.http import HttpResponse, HttpRequest,  HttpResponseNotAllowed, HttpResponseRedirect
from django.shortcuts import render
from gis_csdt.filter_tools import filter_request, neighboring_points
from gis_csdt.models import Dataset, MapElement, MapPoint, Tag, MapPolygon, TagIndiv, DataField, DataElement, Observation, ObservationValue, Sensor, DataPoint, PhoneNumber
from gis_csdt.serializers import TagCountSerializer, DatasetSerializer, MapPointSerializer, NewTagSerializer, MapPolygonSerializer, CountPointsSerializer, AnalyzeAreaSerializer, AnalyzeAreaNoValuesSerializer, SensedDataSerializer, DataPointSerializer, SensorSerializer
#import csv
from gis_csdt.serializers import TestSerializer
from django.views.decorators.csrf import csrf_exempt
from twilio.twiml.messaging_response import MessagingResponse

###constants for pagination
PAGINATE_BY_CONST = 100
PAGINATE_BY_PARAM_CONST = 'page_size'
MAX_PAGINATE_BY_CONST = 500
GSM_CHAR_SET = ("@£$¥èéùìòÇ\nØø\rÅåΔ_ΦΓΛΩΠΨΣΘΞ\x1bÆæßÉ !\"#¤%&'()*+,-./0123456789:;<=>?"
               "¡ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÑÜ`¿abcdefghijklmnopqrstuvwxyzäöñüà").decode("utf8")
###classes for pagination
class PaginatedModelViewSet(viewsets.ModelViewSet):
    paginate_by = PAGINATE_BY_CONST
    paginate_by_param = PAGINATE_BY_PARAM_CONST
    max_paginate_by = MAX_PAGINATE_BY_CONST
class PaginatedReadOnlyModelViewSet(viewsets.ReadOnlyModelViewSet):
    paginate_by = PAGINATE_BY_CONST
    paginate_by_param = PAGINATE_BY_PARAM_CONST
    max_paginate_by = MAX_PAGINATE_BY_CONST
########


class SensedDataViewSet(PaginatedModelViewSet):
    paginate_by = PAGINATE_BY_CONST
    paginate_by_param = PAGINATE_BY_PARAM_CONST
    max_paginate_by = MAX_PAGINATE_BY_CONST
    queryset = ObservationValue.objects.filter()
    serializer_class = SensedDataSerializer

    #http://www.django-rest-framework.org/api-guide/permissions
    permission_classes = [permissions.IsAuthenticated]#(permissions.IsAuthenticatedOrReadOnly)

class NewSensorView(PaginatedModelViewSet):
    paginate_by = PAGINATE_BY_CONST
    paginate_by_param = PAGINATE_BY_PARAM_CONST
    max_paginate_by = MAX_PAGINATE_BY_CONST
    queryset = Sensor.objects.filter()
    serializer_class = SensorSerializer

    #http://www.django-rest-framework.org/api-guide/permissions
    permission_classes = [permissions.IsAuthenticated]#(permissions.IsAuthenticatedOrReadOnly)

class SubmitDataPointView(PaginatedModelViewSet):
    paginate_by = PAGINATE_BY_CONST
    paginate_by_param = PAGINATE_BY_PARAM_CONST
    max_paginate_by = MAX_PAGINATE_BY_CONST
    queryset = DataPoint.objects.all()
    serializer_class = DataPointSerializer

    #http://www.django-rest-framework.org/api-guide/permissions
    permission_classes = [permissions.IsAuthenticated]#(permissions.IsAuthenticatedOrReadOnly)


def GSM7ToInt(string):
    pow = 0
    BASE = 128  # GSM-7 code base
    val = 0
    for char in string:
        val += GSM_CHAR_SET.find(char) * (BASE ** pow)
        pow += 1
    return val

def IntToGSM7(value):
    string = ''
    BASE = 128  # GSM-7 code base
    while value >= BASE:
        string += GSM_CHAR_SET[value%BASE]
        value = int(value/BASE)
    string += GSM_CHAR_SET[value%BASE]
    return string

def DataToGSM7(values):
    string = ''
    BASE = 128  # GSM-7 code base
    MAXSIZE = BASE**2  # 2 bytes of GSM7
    string += IntToGSM7(values.pop(0))
    string += IntToGSM7(values.pop(0))
    for value in values:
        if value > MAXSIZE:
            raise
        if value < BASE:
            string+=IntToGSM7(value)+'@'
        else:
            string+=IntToGSM7(value)
    return string

@csrf_exempt
def SMSSubmitDataPointView(request):
    DATASIZE = 2  # 2 GSM-7 chars
    try:
        data = request.POST.get('Body')
        phNum = int(request.POST.get('From'))
        user = PhoneNumber.objects.get(phone_number=phNum).user
        data = [data[i:i+DATASIZE] for i in range (0, len(data), DATASIZE)]
        source = data.pop(0)
        sensor = Sensor.objects.get(pk=GSM7ToInt(source[0]))
        mapPoint = MapPoint.objects.get(pk=GSM7ToInt(source[1]))
        for dataValue in data:
            newData = DataPoint.objects.create(value=GSM7ToInt(dataValue), point=mapPoint, sensor=sensor, user=user)
            newData.save()
    except:
        raise ParseError('Bad format.')
    return HttpResponse(status=204)


class TestView(PaginatedReadOnlyModelViewSet):
    serializer_class = TestSerializer
    model = MapElement

    def get_queryset(self):
        return filter_request(self.request.query_params, 'mapelement')

class PaginatedCSVRenderer (CSVRenderer):
    results_field = 'results'

    def render(self, data, media_type=None, renderer_context=None):
        if not isinstance(data, list):
            data = data.get(self.results_field, [])
        return super(PaginatedCSVRenderer, self).render(data, media_type, renderer_context)

class NewTagViewSet(PaginatedModelViewSet):
    queryset = TagIndiv.objects.filter(tag__approved=True).distinct('tag')
    serializer_class = NewTagSerializer

    #http://www.django-rest-framework.org/api-guide/permissions
    permission_classes = [permissions.AllowAny]#(permissions.IsAuthenticatedOrReadOnly)

class TagCountViewSet(PaginatedReadOnlyModelViewSet):
    serializer_class = TagCountSerializer
    model = Tag

    def get_queryset(self):
        return Tag.objects.filter(approved = True).values('dataset','tag').annotate(num_tags = Count('id'))

class DatasetViewSet(PaginatedReadOnlyModelViewSet):
    queryset = Dataset.objects.all()
    serializer_class = DatasetSerializer

class MapPointViewSet(PaginatedReadOnlyModelViewSet):
    serializer_class = MapPointSerializer
    model = MapPoint

    def get_queryset(self):
        return filter_request(self.request.query_params, 'mappoint')

class MapPolygonViewSet(PaginatedReadOnlyModelViewSet):
    serializer_class = MapPolygonSerializer
    model = MapPolygon

    def get_queryset(self):
        return filter_request(self.request.query_params, 'mappolygon')

class CountPointsInPolygonView(PaginatedReadOnlyModelViewSet):
    renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES + [PaginatedCSVRenderer]
    serializer_class = CountPointsSerializer
    model = MapPolygon
    permission_classes = [permissions.AllowAny]#(permissions.IsAuthenticatedOrReadOnly)

    def get_queryset(self, format=None):
        use_csv = format and format == 'csv'

        #split params by which it applies to
        mappolygon_params = {}
        for (param, result) in self.request.query_params.items():
            if param in ['max_lat','min_lat','max_lon','min_lon']:
                mappolygon_params[param] = result
            elif param == 'poly_dataset':
                mappolygon_params['dataset'] = result
        #now get the querysets
        polygons = filter_request(mappolygon_params,'mappolygon')
        if 'state' in self.request.query_params:
            polygons = polygons.filter(remote_id__startswith=self.request.query_params['state'])
        return polygons

        if False:
            csv_response = HttpResponse(content_type='text/csv')
            filename = 'census_tract_stats'
            for t in tags:
                filename = filename + '_' + t
            csv_response['Content-Disposition'] = 'attachment; filename="' + filename + '.csv"'
            writer = csv.writer(csv_response)

            firstrow = ['polygon_id', polygons[0].dataset.field1_en, polygons[0].dataset.field2_en]
            for tag in tags:
                try:
                    num = int(tag)
                    t = Tag.objects.get(pk = num)
                    t = t.tag
                except:
                    t = tag
                firstrow.append(t + " count")
            if mult_tags:
                if not matchall:
                    firstrow.append(all_tags + " count (match any)")
                firstrow.append(all_tags + " count (match all)")
            for df in datafields:
                firstrow.append(df.field_en)
            writer.writerow(firstrow)

            for poly in polygons:
                row = [poly.remote_id, poly.field1, poly.field2]

                #counts in polygons
                all_tag_filter = points
                if not matchall:
                    for tag in tags:
                        try:
                            num = int(tag)
                            temp = points.filter(tagindiv__tag=num)
                            all_tag_filter = points.filter(tagindiv__tag=num)
                        except:
                            temp = points.filter(tagindiv__tag__tag=tag)
                            all_tag_filter = points.filter(tagindiv__tag__tag=tag)
                        row.append(temp.filter(point__intersects = poly.mpoly).count())
                    print points.filter(point__intersects = poly.mpoly).query
                    if mult_tags:
                        row.append(points.filter(point__intersects = poly.mpoly).count())
                if mult_tags:
                    row.append(all_tag_filter.filter(point__intersects = poly.mpoly).count())

                #get other data
                for df in datafields:
                    data = None
                    if df.field_type == DataField.INTEGER:
                        element = DataElement.objects.filter(datafield = df).filter(mapelement=poly)
                        if element:
                            data = element[0].int_data
                    elif df.field_type == DataField.FLOAT:
                        element = DataElement.objects.filter(datafield = df).filter(mapelement=poly)
                        if element:
                            data = element[0].float_data
                    else:
                        element = DataElement.objects.filter(datafield = df).filter(mapelement=poly)
                        if element:
                            data = element[0].char_data
                    if data:
                        row.append(data)
                    else:
                        row.append('')
                writer.writerow(row)
            return csv_response

class AnalyzeAreaAroundPointView(PaginatedReadOnlyModelViewSet):
    renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES + [PaginatedCSVRenderer]
    serializer_class = AnalyzeAreaSerializer
    model = MapPoint
    permission_classes = [permissions.AllowAny]#(permissions.IsAuthenticatedOrReadOnly)

    def get_queryset(self, format=None):
        #split params by which it applies to
        mappoint_params = {}
        for (param, result) in self.request.query_params.items():
            if param in ['max_lat','min_lat','max_lon','min_lon','dataset','tags','tag']:
                mappoint_params[param] = result
        #if none of this is specified, this is just too much
        if len(mappoint_params) == 0:
            raise ParseError('Too many results. Please restrict the mappoints.')
        #now get the queryset
        points = filter_request(mappoint_params,'mappoint')
        distances = self.request.GET.getlist('distance')
        unit = self.request.GET.getlist('unit')
        if len(unit) > 1:
            raise ParseError('No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m','km','mi']:
            unit = unit[0]
        else:
            raise ParseError('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1,3,5]
            unit = 'km'
        else:
            distances.sort()
        kwargs = {unit:distances[-1]}
        take_out = []
        for point in points:
            if point.id in take_out:
                continue
            take_out.extend(neighboring_points(point, points, Distance(**kwargs)).exclude(id=point.id).values_list('id',flat=True))
        return points.exclude(id__in=take_out)

class AnalyzeAreaAroundPointNoValuesView(PaginatedReadOnlyModelViewSet):
    serializer_class = AnalyzeAreaNoValuesSerializer
    model = MapPoint
    permission_classes = [permissions.IsAuthenticated]#(permissions.IsAuthenticatedOrReadOnly)

    def get_queryset(self):
        #split params by which it applies to
        mappoint_params = {}
        for (param, result) in self.request.query_params.items():
            if param in ['max_lat','min_lat','max_lon','min_lon','dataset','tags','tag','state','zipcode']:
                mappoint_params[param] = result
        #if none of this is specified, this is just too much
        if len(mappoint_params) == 0:
            raise ParseError('Too many results. Please restrict the mappoints.')
        #now get the queryset
        points = filter_request(mappoint_params,'mappoint')
        distances = self.request.GET.getlist('distance')
        unit = self.request.GET.getlist('unit')
        if len(unit) > 1:
            raise ParseError('No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m','km','mi']:
            unit = unit[0]
        else:
            raise ParseError('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1,3,5]
            unit = 'km'
        else:
            print distances
            distances.sort()
        kwargs = {unit:distances[-1]}
        take_out = []
        for point in points:
            if point.id in take_out:
                continue
            take_out.extend(neighboring_points(point, points, Distance(**kwargs)).exclude(id=point.id).values_list('id',flat=True))
        return points.exclude(id__in=take_out).distinct()



from django.template import Template, Context
from gis_csdt.settings import GOOGLE_API_KEY
from gis_csdt.models import MapPoint
from gis_csdt.geometry_tools import circle_as_polygon
from gis_csdt.filter_tools import neighboring_points
from django.http import HttpResponse
from django.contrib.sites.models import Site
from django.contrib.sites.shortcuts import get_current_site
from django.contrib.gis.measure import Distance
from django.contrib.gis.geos import GEOSGeometry, LineString, Polygon

def get_distances(input):
	try:
		return [float(i.strip()) for i in input.split(',')]
	except:
		return [1,3,5]

def AroundPointView(request, mappoint_id = None):
	current_site = get_current_site(request) #Site.objects.get_current()
	context = {'key': GOOGLE_API_KEY,
			   'zoom_level': 12,
			   'lat': 1,
			   'lon': 45,
			   'width': 500,
			   'height': 380,
			   'root': 'http://'+ current_site.domain + '/'}
	if 'lat' in request.GET:
		try:
			context['lat'] = float(request.GET['lat'])
		except:
			pass
	if 'lon' in request.GET:
		try:
			context['lon'] = float(request.GET['lon'])
		except:
			pass

	if 'distances' in request.GET:
		distances= get_distances(request.GET['distances'])
	elif 'distance' in request.GET:
		distances= get_distances(request.GET['distance'])
	else:
		distances = get_distances(None)

	if 'unit' in request.GET and request.GET['unit'] in ['mi', 'm']:
		unit = request.GET['unit']
	else:
		unit = 'km'

	dataset = 0

	if mappoint_id:
		try:
			mappoint = MapPoint.objects.get(id = mappoint_id)
			if mappoint.point.x != 0:
				context['lon'] = mappoint.point.x
			if mappoint.point.y != 0:
				context['lat'] = mappoint.point.y
			points=[(a.point.y,a.point.x,a.name) for a in neighboring_points(mappoint,MapPoint.objects.filter(dataset=mappoint.dataset_id),Distance(km=distances[-1]))]
			dataset = mappoint.dataset_id
		except Exception as e:
			points=[(context['lat'],context['lon'],"")]
	else:
		points=[(context['lat'],context['lon'],"")]
	print context['root']
	if context['root'] == 'http://example.com/':
		context['root'] = 'http://127.0.0.1:8000/'
	context['circle']=''
	
	for d in distances:
		text = ''
		dist_kwargs = {unit:d}
		dist = Distance(**dist_kwargs)
		geometry = circle_as_polygon(lat = points[0][0], lon = points[0][1], distance = dist)
		for p in points[1:]:
			geometry = geometry.union(circle_as_polygon(lat = p[0], lon = p[1], distance = dist))
		for poly in geometry:
			text = ''
			for ring in poly: 
				if type(ring) is tuple:
					text = text + ', new google.maps.LatLng(%f, %f)\n' %(ring[1], ring[0])
				else:
					for coord in ring:
						text = text + ', new google.maps.LatLng(%f, %f)\n' %(coord[1], coord[0])
			context['circle'] = context['circle'] + ',['+text.strip(',')+']'
	context['circle'] = '[' + context['circle'].strip(',') + ']'
	context['points'] = str(['new google.maps.LatLng(%f,%f)'%(a[0],a[1]) for a in points]).replace("'","")
	context['names'] = str([a[2] for a in points]).replace("u'","'")
	context = Context(context)
	template = """<!DOCTYPE HTML>
	<script src="http://ajax.googleapis.com/ajax/libs/jquery/1.11.1/jquery.min.js"></script>
	<script src="http://maps.googleapis.com/maps/api/js?key={{ key }}&sensor=false">
	</script>

	<script>
	var map;
	function initialize() {
		function add_polygons(){
		  var polygon = new google.maps.Polygon({
		    paths: {{ circle }},
		    strokeColor: '#0033FF',
		    strokeOpacity: 0.8,
		    strokeWeight: 2,
		    fillColor: '#FF0000',
		    fillOpacity: 0.35
		  });
		  polygon.setMap(map);
		}
		function set_height(){
			var h = $(window).height();
		$('#googleMap').css('height', h);
		}
		$(window).resize(set_height());
		set_height();
		// Create a simple map.
		map = new google.maps.Map(document.getElementById('googleMap'), {
			zoom: {{ zoom_level }},
			center: {lat: {{ lat }}, lng:{{ lon }}}
		});
  		add_polygons();
	  // Load a GeoJSON from the same server as our demo.
	  google.maps.event.addListener(map, 'bounds_changed', function() {
	  	load_polys("{{ root }}api-poly/?max_lat=" + map.getBounds().getNorthEast().lat() + "&max_lon=" + map.getBounds().getNorthEast().lng() + "&min_lat=" + map.getBounds().getSouthWest().lat() + "&min_lon="+ map.getBounds().getSouthWest().lng());
  		add_markers("{{ root }}api-mp/?dataset=" + {{ dataset }} +"&max_lat=" + map.getBounds().getNorthEast().lat() + "&max_lon=" + map.getBounds().getNorthEast().lng() + "&min_lat=" + map.getBounds().getSouthWest().lat() + "&min_lon="+ map.getBounds().getSouthWest().lng());


		function add_markers(json_next) {
			$.getJSON(json_next)
				.done(function(data){
				  	for (var i=0;i<data.results.length;i++){
				  		var marker = new google.maps.Marker({
						    position: new google.maps.LatLng(data.results[i].latitude, data.results[i].longitude),
						    title: data.results[i].name
						});
						marker.setMap(map);
				  	}

					json_next = data.next;
					if (data.next != null){
						load_polys(data.next);
					} 
					
				});
		  	var centers = {{ points }};
		  	var names = {{ names|safe }};
		  	for (var i=0;i<centers.length;i++){
		  		var marker = new google.maps.Marker({
				    position: centers[i],
				    title: names[i]
				});
				marker.setMap(map);
		  	}
		}
		
		function load_polys(json_next) {
			$.getJSON(json_next)
			.done(function(data){
				map.data.addGeoJson(data.results);
				json_next = data.next;
				if (data.next != null){
					load_polys(data.next);
				} 
				
			});
		}	
		var featureStyle = {
    fillOpacity: 0,
    strokeWeight: 1
  }
  map.data.setStyle(featureStyle);

	  });

	}

	$( document ).ready(google.maps.event.addDomListener(window, 'load', initialize));

	</script>
	<div id="googleMap" style="width:100%;height:0px;"></div>
"""
	template = Template(template)
	return HttpResponse(template.render(context))